@@ -306,6 +306,11 @@ gst_pulsering_context_state_cb (pa_context * c, void *userdata)
   state = pa_context_get_state (c);
   GST_LOG_OBJECT (psink, "got new context state %d", state);
 
+  /* psink can be null when we are shutting down and the ringbuffer is already
+   * unparented */
+  if (psink == NULL)
+    return;
+
   switch (state) {
     case PA_CONTEXT_READY:
     case PA_CONTEXT_TERMINATED:
@@ -1900,6 +1905,7 @@ gst_pulsesink_get_mute (GstPulseSink * psink)
   gboolean mute = FALSE;
 
   pa_threaded_mainloop_lock (psink->mainloop);
+  mute = psink->mute;
 
   pbuf = GST_PULSERING_BUFFER_CAST (GST_BASE_AUDIO_SINK (psink)->ringbuffer);
   if (pbuf == NULL || pbuf->stream == NULL)
@@ -1918,8 +1924,6 @@ gst_pulsesink_get_mute (GstPulseSink * psink)
       goto unlock;
   }
 
-  mute = psink->mute;
-
 unlock:
   if (o)
     pa_operation_unref (o);
@@ -2227,8 +2231,8 @@ gst_pulsesink_event (GstBaseSink * sink, GstEvent * event)
         gst_tag_list_get_string (l, GST_TAG_PERFORMER, &artist);
 
       if (title && artist)
-        t = buf =
-            g_strdup_printf (_("'%s' by '%s'"), g_strstrip (title),
+        /* TRANSLATORS: 'song title' by 'artist name' */
+        t = buf = g_strdup_printf (_("'%s' by '%s'"), g_strstrip (title),
             g_strstrip (artist));
       else if (title)
         t = g_strstrip (title);
