@@ -24,19 +24,17 @@
 /**
  * SECTION:element-wavpackparse
  *
- * <refsect2>
  * WavpackParse takes raw, unframed Wavpack streams and splits them into
  * single Wavpack chunks with information like bit depth and the position
  * in the stream.
  * <ulink url="http://www.wavpack.com/">Wavpack</ulink> is an open-source
  * audio codec that features both lossless and lossy encoding.
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=test.wv ! wavpackparse ! wavpackdec ! fakesink
- * </programlisting>
- * This pipeline decodes the Wavpack file test.wv into raw audio buffers.
- * </para>
+ * ]| This pipeline decodes the Wavpack file test.wv into raw audio buffers.
  * </refsect2>
  */
 
@@ -798,9 +796,9 @@ gst_wavpack_parse_pull_buffer (GstWavpackParse * wvparse, gint64 offset,
 
   GstBuffer *buf = NULL;
 
-  if (offset + size >= wvparse->upstream_length) {
+  if (offset + size > wvparse->upstream_length) {
     wvparse->upstream_length = gst_wavpack_parse_get_upstream_length (wvparse);
-    if (offset + size >= wvparse->upstream_length) {
+    if (offset + size > wvparse->upstream_length) {
       GST_DEBUG_OBJECT (wvparse, "EOS: %" G_GINT64_FORMAT " + %u > %"
           G_GINT64_FORMAT, offset, size, wvparse->upstream_length);
       flow_ret = GST_FLOW_UNEXPECTED;
@@ -952,6 +950,7 @@ static GstFlowReturn
 gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
     WavpackHeader * header)
 {
+  GstFlowReturn ret;
   wvparse->current_offset += header->ckSize + 8;
 
   wvparse->segment.last_stop = header->block_index;
@@ -1009,7 +1008,11 @@ gst_wavpack_parse_push_buffer (GstWavpackParse * wvparse, GstBuffer * buf,
   GST_LOG_OBJECT (wvparse, "Pushing buffer with time %" GST_TIME_FORMAT,
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
 
-  return gst_pad_push (wvparse->srcpad, buf);
+  ret = gst_pad_push (wvparse->srcpad, buf);
+
+  wvparse->segment.last_stop = wvparse->next_block_index;
+
+  return ret;
 }
 
 static guint8 *
