@@ -55,9 +55,6 @@ static GstStaticPadTemplate gst_rtp_mpa_pay_src_template =
         "clock-rate = (int) 90000, " "encoding-name = (string) \"MPA\"")
     );
 
-static void gst_rtp_mpa_pay_class_init (GstRtpMPAPayClass * klass);
-static void gst_rtp_mpa_pay_base_init (GstRtpMPAPayClass * klass);
-static void gst_rtp_mpa_pay_init (GstRtpMPAPay * rtpmpapay);
 static void gst_rtp_mpa_pay_finalize (GObject * object);
 
 static gboolean gst_rtp_mpa_pay_setcaps (GstBaseRTPPayload * payload,
@@ -65,35 +62,10 @@ static gboolean gst_rtp_mpa_pay_setcaps (GstBaseRTPPayload * payload,
 static GstFlowReturn gst_rtp_mpa_pay_handle_buffer (GstBaseRTPPayload * payload,
     GstBuffer * buffer);
 
-static GstBaseRTPPayloadClass *parent_class = NULL;
+GST_BOILERPLATE (GstRtpMPAPay, gst_rtp_mpa_pay, GstBaseRTPPayload,
+    GST_TYPE_BASE_RTP_PAYLOAD)
 
-static GType
-gst_rtp_mpa_pay_get_type (void)
-{
-  static GType rtpmpapay_type = 0;
-
-  if (!rtpmpapay_type) {
-    static const GTypeInfo rtpmpapay_info = {
-      sizeof (GstRtpMPAPayClass),
-      (GBaseInitFunc) gst_rtp_mpa_pay_base_init,
-      NULL,
-      (GClassInitFunc) gst_rtp_mpa_pay_class_init,
-      NULL,
-      NULL,
-      sizeof (GstRtpMPAPay),
-      0,
-      (GInstanceInitFunc) gst_rtp_mpa_pay_init,
-    };
-
-    rtpmpapay_type =
-        g_type_register_static (GST_TYPE_BASE_RTP_PAYLOAD, "GstRtpMPAPay",
-        &rtpmpapay_info, 0);
-  }
-  return rtpmpapay_type;
-}
-
-static void
-gst_rtp_mpa_pay_base_init (GstRtpMPAPayClass * klass)
+     static void gst_rtp_mpa_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
@@ -114,8 +86,6 @@ gst_rtp_mpa_pay_class_init (GstRtpMPAPayClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->finalize = gst_rtp_mpa_pay_finalize;
 
   gstbasertppayload_class->set_caps = gst_rtp_mpa_pay_setcaps;
@@ -123,7 +93,7 @@ gst_rtp_mpa_pay_class_init (GstRtpMPAPayClass * klass)
 }
 
 static void
-gst_rtp_mpa_pay_init (GstRtpMPAPay * rtpmpapay)
+gst_rtp_mpa_pay_init (GstRtpMPAPay * rtpmpapay, GstRtpMPAPayClass * klass)
 {
   rtpmpapay->adapter = gst_adapter_new ();
 }
@@ -198,7 +168,7 @@ gst_rtp_mpa_pay_flush (GstRtpMPAPay * rtpmpapay)
      *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
      * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      * |             MBZ               |          Frag_offset          |
-     * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
+     * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
      */
     payload = gst_rtp_buffer_get_payload (outbuf);
     payload[0] = 0;
@@ -245,7 +215,7 @@ gst_rtp_mpa_pay_handle_buffer (GstBaseRTPPayload * basepayload,
     rtpmpapay->duration = 0;
   }
 
-  /* get packet length of previous data and this new data, 
+  /* get packet length of previous data and this new data,
    * payload length includes a 4 byte header */
   packet_len = gst_rtp_buffer_calc_packet_len (4 + avail + size, 0, 0);
 
