@@ -32,7 +32,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch -v videotestsrc ! video/x-raw-yuv,format=(fourcc)AYUV,width=640,height=480 ! shapewipe position=0.5 name=shape ! videomixer name=mixer ! ffmpegcolorspace ! autovideosink     filesrc location=mask.png ! typefind ! decodebin2 ! ffmpegcolorspace ! videoscale ! queue ! shape.mask_sink    videotestsrc pattern=snow ! video/x-raw-yuv,format=(fourcc)AYUV,width=640,height=480 ! queue ! mixer.
+ * gst-launch -v videotestsrc ! video/x-raw-yuv,width=640,height=480 ! shapewipe position=0.5 name=shape ! videomixer name=mixer ! ffmpegcolorspace ! autovideosink     filesrc location=mask.png ! typefind ! decodebin2 ! ffmpegcolorspace ! videoscale ! queue ! shape.mask_sink    videotestsrc pattern=snow ! video/x-raw-yuv,width=640,height=480 ! queue ! mixer.
  * ]| This pipeline adds the transition from mask.png with position 0.5 to an SMPTE test screen and snow.
  * </refsect2>
  */
@@ -100,7 +100,7 @@ static GstStaticPadTemplate video_sink_pad_template =
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV") " ; " GST_VIDEO_CAPS_ARGB " ; "
-        GST_VIDEO_CAPS_BGRA ";" GST_VIDEO_CAPS_ABGR ";" GST_VIDEO_CAPS_RGBA));
+        GST_VIDEO_CAPS_BGRA));
 
 static GstStaticPadTemplate mask_sink_pad_template =
     GST_STATIC_PAD_TEMPLATE ("mask_sink",
@@ -118,7 +118,7 @@ static GstStaticPadTemplate mask_sink_pad_template =
 static GstStaticPadTemplate src_pad_template =
     GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV") " ; " GST_VIDEO_CAPS_ARGB " ; "
-        GST_VIDEO_CAPS_BGRA ";" GST_VIDEO_CAPS_ABGR ";" GST_VIDEO_CAPS_RGBA));
+        GST_VIDEO_CAPS_BGRA));
 
 GST_DEBUG_CATEGORY_STATIC (gst_shape_wipe_debug);
 #define GST_CAT_DEFAULT gst_shape_wipe_debug
@@ -136,12 +136,12 @@ gst_shape_wipe_base_init (gpointer g_class)
       "Adds a shape wipe transition to a video stream",
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &video_sink_pad_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &mask_sink_pad_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &src_pad_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&video_sink_pad_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&mask_sink_pad_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_pad_template));
 }
 
 static void
@@ -385,14 +385,7 @@ gst_shape_wipe_video_sink_getcaps (GstPad * pad)
     ret = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
   }
 
-  GST_LOG_OBJECT (pad, "srcpad accepted caps: %" GST_PTR_FORMAT, ret);
-
-  if (gst_caps_is_empty (ret))
-    goto done;
-
   tmp = gst_pad_peer_get_caps (pad);
-
-  GST_LOG_OBJECT (pad, "peerpad accepted caps: %" GST_PTR_FORMAT, tmp);
   if (tmp) {
     GstCaps *intersection;
 
@@ -402,11 +395,6 @@ gst_shape_wipe_video_sink_getcaps (GstPad * pad)
     ret = intersection;
   }
 
-  GST_LOG_OBJECT (pad, "intersection: %" GST_PTR_FORMAT, tmp);
-
-  if (gst_caps_is_empty (ret))
-    goto done;
-
   if (self->height && self->width) {
     guint i, n;
 
@@ -420,8 +408,6 @@ gst_shape_wipe_video_sink_getcaps (GstPad * pad)
   }
 
   tmp = gst_pad_peer_get_caps (self->mask_sinkpad);
-
-  GST_LOG_OBJECT (pad, "mask accepted caps: %" GST_PTR_FORMAT, tmp);
   if (tmp) {
     GstCaps *intersection, *tmp2;
     guint i, n;
@@ -457,7 +443,6 @@ gst_shape_wipe_video_sink_getcaps (GstPad * pad)
     gst_caps_unref (ret);
     ret = intersection;
   }
-done:
 
   gst_object_unref (self);
 
@@ -524,14 +509,7 @@ gst_shape_wipe_mask_sink_getcaps (GstPad * pad)
     ret = gst_caps_copy (gst_pad_get_pad_template_caps (self->video_sinkpad));
   }
 
-  GST_LOG_OBJECT (pad, "video sink accepted caps: %" GST_PTR_FORMAT, ret);
-
-  if (gst_caps_is_empty (ret))
-    goto done;
-
   tmp = gst_pad_peer_get_caps (self->srcpad);
-  GST_LOG_OBJECT (pad, "srcpad accepted caps: %" GST_PTR_FORMAT, ret);
-
   if (tmp) {
     GstCaps *intersection;
 
@@ -541,11 +519,6 @@ gst_shape_wipe_mask_sink_getcaps (GstPad * pad)
     ret = intersection;
   }
 
-  GST_LOG_OBJECT (pad, "intersection: %" GST_PTR_FORMAT, ret);
-
-  if (gst_caps_is_empty (ret))
-    goto done;
-
   n = gst_caps_get_size (ret);
   tmp = gst_caps_new_empty ();
   for (i = 0; i < n; i++) {
@@ -574,8 +547,6 @@ gst_shape_wipe_mask_sink_getcaps (GstPad * pad)
   gst_caps_append (ret, tmp);
 
   tmp = gst_pad_peer_get_caps (pad);
-  GST_LOG_OBJECT (pad, "peer accepted caps: %" GST_PTR_FORMAT, tmp);
-
   if (tmp) {
     GstCaps *intersection;
 
@@ -585,7 +556,6 @@ gst_shape_wipe_mask_sink_getcaps (GstPad * pad)
     ret = intersection;
   }
 
-done:
   gst_object_unref (self);
 
   GST_LOG_OBJECT (pad, "Returning caps: %" GST_PTR_FORMAT, ret);
@@ -614,13 +584,7 @@ gst_shape_wipe_src_getcaps (GstPad * pad)
     ret = gst_caps_copy (gst_pad_get_pad_template_caps (self->video_sinkpad));
   }
 
-  GST_LOG_OBJECT (pad, "video sink accepted caps: %" GST_PTR_FORMAT, ret);
-
-  if (gst_caps_is_empty (ret))
-    goto done;
-
   tmp = gst_pad_peer_get_caps (pad);
-  GST_LOG_OBJECT (pad, "peer accepted caps: %" GST_PTR_FORMAT, ret);
   if (tmp) {
     GstCaps *intersection;
 
@@ -630,11 +594,6 @@ gst_shape_wipe_src_getcaps (GstPad * pad)
     ret = intersection;
   }
 
-  GST_LOG_OBJECT (pad, "intersection: %" GST_PTR_FORMAT, ret);
-
-  if (gst_caps_is_empty (ret))
-    goto done;
-
   if (self->height && self->width) {
     guint i, n;
 
@@ -648,7 +607,6 @@ gst_shape_wipe_src_getcaps (GstPad * pad)
   }
 
   tmp = gst_pad_peer_get_caps (self->mask_sinkpad);
-  GST_LOG_OBJECT (pad, "mask sink accepted caps: %" GST_PTR_FORMAT, ret);
   if (tmp) {
     GstCaps *intersection, *tmp2;
     guint i, n;
@@ -684,8 +642,6 @@ gst_shape_wipe_src_getcaps (GstPad * pad)
     ret = intersection;
   }
 
-done:
-
   gst_object_unref (self);
 
   GST_LOG_OBJECT (pad, "Returning caps: %" GST_PTR_FORMAT, ret);
@@ -810,6 +766,75 @@ gst_shape_wipe_do_qos (GstShapeWipe * self, GstClockTime timestamp)
   return TRUE;
 }
 
+#define CREATE_AYUV_FUNCTIONS(depth, shift) \
+static void \
+gst_shape_wipe_blend_ayuv_##depth (GstShapeWipe * self, GstBuffer * inbuf, \
+    GstBuffer * maskbuf, GstBuffer * outbuf) \
+{ \
+  const guint##depth *mask = (const guint##depth *) GST_BUFFER_DATA (maskbuf); \
+  const guint8 *input = (const guint8 *) GST_BUFFER_DATA (inbuf); \
+  guint8 *output = (guint8 *) GST_BUFFER_DATA (outbuf); \
+  guint i, j; \
+  guint mask_increment = ((depth == 16) ? GST_ROUND_UP_2 (self->width) : \
+                           GST_ROUND_UP_4 (self->width)) - self->width; \
+  gfloat position = self->mask_position; \
+  gfloat low = position - (self->mask_border / 2.0f); \
+  gfloat high = position + (self->mask_border / 2.0f); \
+  guint32 low_i, high_i, position_i, round_i; \
+  gint width = self->width, height = self->height; \
+  \
+  if (low < 0.0f) { \
+    high = 0.0f; \
+    low = 0.0f; \
+  } \
+  \
+  if (high > 1.0f) { \
+    low = 1.0f; \
+    high = 1.0f; \
+  } \
+  \
+  position_i = position * 65536; \
+  low_i = low * 65536; \
+  high_i = high * 65536; \
+  round_i = (high_i - low_i) >> 1; \
+  \
+  for (i = 0; i < height; i++) { \
+    for (j = 0; j < width; j++) { \
+      guint32 in = *mask << shift; \
+      \
+      if (in < low_i) { \
+        output[0] = 0x00;       /* A */ \
+        output[1] = input[1];   /* Y */ \
+        output[2] = input[2];   /* U */ \
+        output[3] = input[3];   /* V */ \
+      } else if (in >= high_i) { \
+        output[0] = input[0];   /* A */ \
+        output[1] = input[1];   /* Y */ \
+        output[2] = input[2];   /* U */ \
+        output[3] = input[3];   /* V */ \
+      } else { \
+        guint32 val; \
+        /* Note: This will never overflow or be larger than 255! */ \
+        val = (((in - low_i) << 16) + round_i) / (high_i - low_i); \
+        val = (val * input[0] + 32768) >> 16; \
+        \
+        output[0] = val;        /* A */ \
+        output[1] = input[1];   /* Y */ \
+        output[2] = input[2];   /* U */ \
+        output[3] = input[3];   /* V */ \
+      } \
+      \
+      mask++; \
+      input += 4; \
+      output += 4; \
+    } \
+    mask += mask_increment; \
+  } \
+}
+
+CREATE_AYUV_FUNCTIONS (16, 0);
+CREATE_AYUV_FUNCTIONS (8, 8);
+
 #define CREATE_ARGB_FUNCTIONS(depth, name, shift, a, r, g, b) \
 static void \
 gst_shape_wipe_blend_##name##_##depth (GstShapeWipe * self, GstBuffer * inbuf, \
@@ -824,7 +849,7 @@ gst_shape_wipe_blend_##name##_##depth (GstShapeWipe * self, GstBuffer * inbuf, \
   gfloat position = self->mask_position; \
   gfloat low = position - (self->mask_border / 2.0f); \
   gfloat high = position + (self->mask_border / 2.0f); \
-  guint32 low_i, high_i, round_i; \
+  guint32 low_i, high_i, position_i, round_i; \
   gint width = self->width, height = self->height; \
   \
   if (low < 0.0f) { \
@@ -837,6 +862,7 @@ gst_shape_wipe_blend_##name##_##depth (GstShapeWipe * self, GstBuffer * inbuf, \
     high = 1.0f; \
   } \
   \
+  position_i = position * 65536; \
   low_i = low * 65536; \
   high_i = high * 65536; \
   round_i = (high_i - low_i) >> 1; \
@@ -936,26 +962,20 @@ gst_shape_wipe_video_sink_chain (GstPad * pad, GstBuffer * buffer)
     outbuf = buffer;
   }
 
-  switch (self->fmt) {
-    case GST_VIDEO_FORMAT_AYUV:
-    case GST_VIDEO_FORMAT_ARGB:
-    case GST_VIDEO_FORMAT_ABGR:
-      if (self->mask_bpp == 16)
-        gst_shape_wipe_blend_argb_16 (self, buffer, mask, outbuf);
-      else
-        gst_shape_wipe_blend_argb_8 (self, buffer, mask, outbuf);
-      break;
-    case GST_VIDEO_FORMAT_BGRA:
-    case GST_VIDEO_FORMAT_RGBA:
-      if (self->mask_bpp == 16)
-        gst_shape_wipe_blend_bgra_16 (self, buffer, mask, outbuf);
-      else
-        gst_shape_wipe_blend_bgra_8 (self, buffer, mask, outbuf);
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
-  }
+  if (self->fmt == GST_VIDEO_FORMAT_AYUV && self->mask_bpp == 16)
+    gst_shape_wipe_blend_ayuv_16 (self, buffer, mask, outbuf);
+  else if (self->fmt == GST_VIDEO_FORMAT_AYUV)
+    gst_shape_wipe_blend_ayuv_8 (self, buffer, mask, outbuf);
+  else if (self->fmt == GST_VIDEO_FORMAT_ARGB && self->mask_bpp == 16)
+    gst_shape_wipe_blend_argb_16 (self, buffer, mask, outbuf);
+  else if (self->fmt == GST_VIDEO_FORMAT_ARGB)
+    gst_shape_wipe_blend_argb_8 (self, buffer, mask, outbuf);
+  else if (self->fmt == GST_VIDEO_FORMAT_BGRA && self->mask_bpp == 16)
+    gst_shape_wipe_blend_bgra_16 (self, buffer, mask, outbuf);
+  else if (self->fmt == GST_VIDEO_FORMAT_BGRA)
+    gst_shape_wipe_blend_bgra_8 (self, buffer, mask, outbuf);
+  else
+    g_assert_not_reached ();
 
   gst_buffer_unref (mask);
   if (new_outbuf)
